body {
    background-color: #222;
    color: white;
}

button {
    color: black;
}

h1, h2, h3 {
    color: #2a2;
}

canvas {
    border: 1px solid #2a2;
    margin-bottom: 1em;
}

#editor {
    border-color: #2a2;
}

.tab-pane {
    margin: 1em;
}

button:disabled {
    color: grey;
}

pre, code {
    color: #9fff8f;
    background-color: #115215;
}

#debugTable th {
    border-bottom: 1px solid #2a2;
}

#debugTable td {
    width: 100px;
}

#debugTable tr th:first-child,td:first-child {
    border-right: 1px solid #2a2;
}